@@ -1,7 +1,7 @@
 /*******************************************************************************
    This file is part of LibTMCG.
 
- Copyright (C) 2004, 2005, 2006  Heiko Stamer <stamer@gaos.org>
+ Copyright (C) 2004, 2005, 2006, 2016  Heiko Stamer <HeikoStamer@gmx.net>
 
    LibTMCG is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,6 +18,10 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/
 
+// include headers
+#ifdef HAVE_CONFIG_H
+	#include "libTMCG_config.h"
+#endif
 #include "TMCG_CardSecret.hh"
 
 TMCG_CardSecret::TMCG_CardSecret
@@ -100,13 +104,15 @@ bool TMCG_CardSecret::import
 		// public card data
 		if (TMCG_ParseHelper::gs(s, '|').length() == 0)
 			throw false;
-		size_t k = strtoul(TMCG_ParseHelper::gs(s, '|').c_str(), &ec, 10);
+		size_t k = 
+		    std::strtoul(TMCG_ParseHelper::gs(s, '|').c_str(), &ec, 10);
 		if ((*ec != '\0') || (k < 1) || (k > TMCG_MAX_PLAYERS) || 
 			(!TMCG_ParseHelper::nx(s, '|')))
 				throw false;
 		if (TMCG_ParseHelper::gs(s, '|').length() == 0)
 			throw false;
-		size_t w = strtoul(TMCG_ParseHelper::gs(s, '|').c_str(), &ec, 10);
+		size_t w = 
+		    std::strtoul(TMCG_ParseHelper::gs(s, '|').c_str(), &ec, 10);
 		if ((*ec != '\0') || (w < 1) || (w > TMCG_MAX_TYPEBITS) || 
 			(!TMCG_ParseHelper::nx(s, '|')))
 				throw false;
