@@ -1,20 +1,23 @@
 /*******************************************************************************
    This file is part of LibTMCG.
 
-     Christian Schindelhauer: 'A Toolbox for Mental Card Games',
+     [Sc98] Christian Schindelhauer: 'A Toolbox for Mental Card Games',
      Technical Report A-98-14, University of L{\"u}beck, 1998.
 
-     Rosario Gennaro, Daniele Micciancio, Tal Rabin:
+     [GMR98] Rosario Gennaro, Daniele Micciancio, Tal Rabin:
      'An Efficient Non-Interactive Statistical Zero-Knowledge
       Proof System for Quasi-Safe Prime Products',
-     5th ACM Conference on Computer and Communication Security, 1998
+     5th ACM Conference on Computer and Communication Security, 1998.
 
-     Mihir Bellare, Phillip Rogaway: 'The Exact Security of Digital
-      Signatures -- How to Sign with RSA and Rabin', 1996
+     [BR96] Mihir Bellare, Phillip Rogaway: 'The Exact Security of Digital
+      Signatures -- How to Sign with RSA and Rabin',
+     Advances in Cryptology - Eurocrypt 96 Proceedings, LNCS 1070, 1996.
 
-     Dan Boneh: 'Simplified OAEP for the RSA and Rabin Functions', 2002
+     [Bo01] Dan Boneh: 'Simplified OAEP for the RSA and Rabin Functions',
+     Proceedings of CRYPTO 2001, LNCS 2139, pp. 275--291, 2001.
 
- Copyright (C) 2004, 2005, 2006, 2007  Heiko Stamer <stamer@gaos.org>
+ Copyright (C) 2004, 2005, 2006, 2007, 
+                           2016, 2017  Heiko Stamer <HeikoStamer@gmx.net>
 
    LibTMCG is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,6 +34,10 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/
 
+// include headers
+#ifdef HAVE_CONFIG_H
+	#include "libTMCG_config.h"
+#endif
 #include "TMCG_PublicKey.hh"
 
 TMCG_PublicKey::TMCG_PublicKey
@@ -82,7 +89,8 @@ bool TMCG_PublicKey::check
 	std::string s = nizk;
 	size_t stage1_size = 0, stage2_size = 0, stage3_size = 0;
 	size_t mnsize = mpz_sizeinbase(m, 2L) / 8;
-	char *ec, *mn = new char[mnsize];
+	char *ec;
+	unsigned char *mn = new unsigned char[mnsize];
 	
 	mpz_init(foo), mpz_init(bar);
 	try
@@ -97,7 +105,7 @@ bool TMCG_PublicKey::check
 		
 		// sanity check, whether m \not\in P (prime)
 		// (here is a very small probability of false-negative behaviour,
-		// FIXME: give a short witness in public key)
+		// FIXME: give a short witness for compositeness in public key)
 		if (mpz_probab_prime_p(m, 500))
 			throw false;
 		
@@ -111,17 +119,14 @@ bool TMCG_PublicKey::check
 		// check, whether m \not\in FP (fermat primes: m = 2^k + 1)
 		mpz_set(foo, m);
 		mpz_sub_ui(foo, foo, 1L);
-// FIXME: is m correct here? (not foo) 
 		unsigned long int k = mpz_sizeinbase(m, 2L);
-		mpz_set_ui(bar, 2L);
-		mpz_pow_ui(bar, bar, k);
+		mpz_ui_pow_ui(bar, 2L, k);
 		if (!mpz_cmp(foo, bar))
 		{
 			// check, whether k is power of two
 			mpz_set_ui(foo, k);
 			unsigned long int l = mpz_sizeinbase(foo, 2L);
-			mpz_set_ui(bar, 2L);
-			mpz_pow_ui(bar, bar, l);
+			mpz_ui_pow_ui(bar, 2L, l);
 			if (!mpz_cmp(foo, bar))
 			{
 				// check, whether m is not equal to 5L
@@ -129,8 +134,7 @@ bool TMCG_PublicKey::check
 					throw false;
 				
 				// check, whether 5^{2^(k/2)} \equiv -1 (mod m) [Pepin's prime test]
-				mpz_set_ui(foo, 2L);
-				mpz_powm_ui(foo, foo, (k / 2), m);
+				mpz_ui_pow_ui(foo, 2L, (k / 2));
 				mpz_set_ui(bar, 5L);
 				mpz_powm(foo, bar, foo, m);
 				mpz_set_si(bar, -1L);
@@ -170,7 +174,7 @@ bool TMCG_PublicKey::check
 			// common random number foo \in Z^*_m (build from hash function g)
 			do
 			{
-				g(mn, mnsize, (input.str()).c_str(), (input.str()).length());
+				g(mn, mnsize, (unsigned char*)(input.str()).c_str(), (input.str()).length());
 				mpz_import(foo, 1, -1, mnsize, 1, 0, mn);
 				mpz_mod(foo, foo, m);
 				mpz_gcd(bar, foo, m);
@@ -209,7 +213,7 @@ bool TMCG_PublicKey::check
 			// common random number foo \in Z^*_m (build from hash function g)
 			do
 			{
-				g(mn, mnsize, (input.str()).c_str(), (input.str()).length());
+				g(mn, mnsize, (unsigned char*)(input.str()).c_str(), (input.str()).length());
 				mpz_import(foo, 1, -1, mnsize, 1, 0, mn);
 				mpz_mod(foo, foo, m);
 				mpz_gcd(bar, foo, m);
@@ -261,7 +265,7 @@ bool TMCG_PublicKey::check
 			// common random number foo \in Z^\circ_m (build from hash function g)
 			do
 			{
-				g(mn, mnsize, (input.str()).c_str(), (input.str()).length());
+				g(mn, mnsize, (unsigned char*)(input.str()).c_str(), (input.str()).length());
 				mpz_import(foo, 1, -1, mnsize, 1, 0, mn);
 				mpz_mod(foo, foo, m);
 				input << foo;
@@ -302,18 +306,17 @@ std::string TMCG_PublicKey::fingerprint
 	() const
 {
 	unsigned int hash_size = gcry_md_get_algo_dlen(TMCG_GCRY_MD_ALGO);
-	char *digest = new char[hash_size];
+	unsigned char *digest = new unsigned char[hash_size];
 	char *hex_digest = new char[(3 * hash_size) + 1];
 	std::ostringstream data;
 	
 	// compute the digest
 	data << "pub|" << name << "|" << email << "|" << type << 
 		"|" << m << "|" << y << "|" << nizk << "|" << sig;
-	h(digest, data.str().c_str(), data.str().length());
+	h(digest, (unsigned char*)(data.str()).c_str(), (data.str()).length());
 	// convert the digest to a hexadecimal encoded string
-	for (unsigned int i = 0; i < (hash_size / 2); i++)
-		snprintf(hex_digest + (5 * i), 6, "%02X%02X ", 
-			(unsigned char)digest[2 * i], (unsigned char)digest[(2 * i) + 1]);
+	for (size_t i = 0; i < (hash_size / 2); i++)
+		snprintf(hex_digest + (5 * i), 6, "%02X%02X ", digest[2 * i], digest[(2 * i) + 1]);
 	std::string fp_str = hex_digest;
 	// release resources
 	delete [] digest, delete [] hex_digest;
@@ -343,7 +346,7 @@ std::string TMCG_PublicKey::selfid
 }
 
 std::string TMCG_PublicKey::keyid
-	(size_t size) const
+	(const size_t size) const
 {
 	std::ostringstream data;
 	std::string tmp = selfid();
@@ -362,12 +365,11 @@ size_t TMCG_PublicKey::keyid_size
 {
 	// check the format
 	if ((s.length() < 4) || (s.substr(0, 2) != "ID") || (s.find("^") == s.npos))
-			return 0;
+		return 0;
 	
 	// extract the size
 	char *ec;
-	size_t size = 
-	    std::strtoul(s.substr(2, s.find("^") - 2).c_str(), &ec, 10);
+	size_t size = std::strtoul(s.substr(2, s.find("^") - 2).c_str(), &ec, 10);
 	if ((*ec != '\0') || (size != (s.length() - s.find("^") - 1)))
 		return 0;
 	
@@ -440,7 +442,7 @@ bool TMCG_PublicKey::import
 }
 
 std::string TMCG_PublicKey::encrypt
-	(const char* value) const
+	(const unsigned char* value) const
 {
 	mpz_t vdata;
 	size_t rabin_s2 = 2 * TMCG_SAEP_S0;
@@ -450,25 +452,27 @@ std::string TMCG_PublicKey::encrypt
 	assert(rabin_s2 < rabin_s1);
 	assert(TMCG_SAEP_S0 < (mpz_sizeinbase(m, 2L) / 32));
 	
-	char *r = new char[rabin_s1];
-	gcry_randomize((unsigned char*)r, rabin_s1, GCRY_STRONG_RANDOM);
+	// use simplified OAEP = SAEP [Bo01]
+	unsigned char *r = new unsigned char[rabin_s1];
+	gcry_randomize(r, rabin_s1, GCRY_STRONG_RANDOM);
 	
-	char *Mt = new char[rabin_s2], *g12 = new char[rabin_s2];
-	std::memcpy(Mt, value, TMCG_SAEP_S0);
-	std::memset(Mt + TMCG_SAEP_S0, 0, TMCG_SAEP_S0);
+	unsigned char *Mt = new unsigned char[rabin_s2];
+	unsigned char *g12 = new unsigned char[rabin_s2];
+	memcpy(Mt, value, TMCG_SAEP_S0);
+	memset(Mt + TMCG_SAEP_S0, 0, TMCG_SAEP_S0);
 	g(g12, rabin_s2, r, rabin_s1);
 	
 	for (size_t i = 0; i < rabin_s2; i++)
 		Mt[i] ^= g12[i];
 	
-	char *yy = new char[rabin_s2 + rabin_s1];
-	std::memcpy(yy, Mt, rabin_s2);
-	std::memcpy(yy + rabin_s2, r, rabin_s1);
+	unsigned char *yy = new unsigned char[rabin_s2 + rabin_s1];
+	memcpy(yy, Mt, rabin_s2);
+	memcpy(yy + rabin_s2, r, rabin_s1);
 	mpz_init(vdata);
 	mpz_import(vdata, 1, -1, rabin_s2 + rabin_s1, 1, 0, yy);
 	delete [] yy, delete [] g12, delete [] Mt, delete [] r;
 	
-	// apply RABIN function vdata = vdata^2 mod m
+	// apply Rabin's encryption function, i.e., vdata = vdata^2 mod m
 	mpz_mul(vdata, vdata, vdata);
 	mpz_mod(vdata, vdata, m);
 	
@@ -501,40 +505,40 @@ bool TMCG_PublicKey::verify
 			TMCG_MPZ_IO_BASE) < 0) || (!TMCG_ParseHelper::nx(s, '|')))
 				throw false;
 		
-		// verify signature
+		// verify signature (see PRab [BR96])
 		size_t mdsize = gcry_md_get_algo_dlen(TMCG_GCRY_MD_ALGO);
 		size_t mnsize = mpz_sizeinbase(m, 2L) / 8;
-		
-		assert(mpz_sizeinbase(m, 2L) > (mnsize * 8));
-		assert(mnsize > (mdsize + TMCG_PRAB_K0));
-		
-		mpz_mul(foo, foo, foo);
+		if (mpz_sizeinbase(m, 2L) <= (mnsize * 8))
+			throw false;
+		if (mnsize <= (mdsize + TMCG_PRAB_K0))
+			throw false;
+		mpz_mul(foo, foo, foo); // apply Rabin's verification function
 		mpz_mod(foo, foo, m);
-		
-		char *w = new char[mdsize], *r = new char[TMCG_PRAB_K0];
-		char *gamma = new char[mnsize - mdsize - TMCG_PRAB_K0];
-		char *yy = new char[mnsize + 1024];
+		unsigned char *w = new unsigned char[mdsize];
+		unsigned char *r = new unsigned char[TMCG_PRAB_K0];
+		unsigned char *gamma = new unsigned char[mnsize - mdsize - TMCG_PRAB_K0];
+		unsigned char *yy = new unsigned char[mnsize + 1024];
 		size_t cnt = 1;
 		mpz_export(yy, &cnt, -1, mnsize, 1, 0, foo);
-		std::memcpy(w, yy, mdsize);
-		std::memcpy(r, yy + mdsize, TMCG_PRAB_K0);
-		std::memcpy(gamma, yy + mdsize + TMCG_PRAB_K0,
+		memcpy(w, yy, mdsize);
+		memcpy(r, yy + mdsize, TMCG_PRAB_K0);
+		memcpy(gamma, yy + mdsize + TMCG_PRAB_K0,
 			mnsize - mdsize - TMCG_PRAB_K0);
 		
-		char *g12 = new char[mnsize];
+		unsigned char *g12 = new unsigned char[mnsize];
 		g(g12, mnsize - mdsize, w, mdsize);
 		
 		for (size_t i = 0; i < TMCG_PRAB_K0; i++)
 			r[i] ^= g12[i];
 		
-		char *Mr = new char[data.length() + TMCG_PRAB_K0];
-		std::memcpy(Mr, data.c_str(), data.length());
-		std::memcpy(Mr + data.length(), r, TMCG_PRAB_K0);
+		unsigned char *Mr = new unsigned char[data.length() + TMCG_PRAB_K0];
+		memcpy(Mr, data.c_str(), data.length());
+		memcpy(Mr + data.length(), r, TMCG_PRAB_K0);
 		
-		char *w2 = new char[mdsize];
+		unsigned char *w2 = new unsigned char[mdsize];
 		h(w2, Mr, data.length() + TMCG_PRAB_K0);
 		
-		bool ok = (std::memcmp(w, w2, mdsize) == 0) && (std::memcmp(gamma,
+		bool ok = (memcmp(w, w2, mdsize) == 0) && (memcmp(gamma,
 			g12 + TMCG_PRAB_K0, mnsize - mdsize - TMCG_PRAB_K0) == 0);
 		delete [] yy, delete [] w, delete [] r, delete [] gamma, 
 			delete [] g12, delete [] Mr, delete [] w2;
