@@ -1,20 +1,23 @@
 /*******************************************************************************
    This file is part of LibTMCG.
 
-     Christian Schindelhauer: 'A Toolbox for Mental Card Games',
+     [Sc98] Christian Schindelhauer: 'A Toolbox for Mental Card Games',
      Technical Report A-98-14, University of L{\"u}beck, 1998.
 
-     Rosario Gennaro, Daniele Micciancio, Tal Rabin:
+     [GMR98] Rosario Gennaro, Daniele Micciancio, Tal Rabin:
      'An Efficient Non-Interactive Statistical Zero-Knowledge
       Proof System for Quasi-Safe Prime Products',
      5th ACM Conference on Computer and Communication Security, 1998.
 
-     Mihir Bellare, Phillip Rogaway: 'The Exact Security of Digital
-      Signatures -- How to Sign with RSA and Rabin', 1996
+     [BR96] Mihir Bellare, Phillip Rogaway: 'The Exact Security of Digital
+      Signatures -- How to Sign with RSA and Rabin', 
+     Advances in Cryptology - Eurocrypt 96 Proceedings, LNCS 1070, 1996.
 
-     Dan Boneh: 'Simplified OAEP for the RSA and Rabin Functions', 2002
+     [Bo01] Dan Boneh: 'Simplified OAEP for the RSA and Rabin Functions',
+     Proceedings of CRYPTO 2001, LNCS 2139, pp. 275--291, 2001.
 
- Copyright (C) 2004, 2005, 2006, 2007  Heiko Stamer <stamer@gaos.org>
+ Copyright (C) 2004, 2005, 2006, 2007, 
+                           2016, 2017  Heiko Stamer <HeikoStamer@gmx.net>
 
    LibTMCG is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,6 +34,10 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/
 
+// include headers
+#ifdef HAVE_CONFIG_H
+	#include "libTMCG_config.h"
+#endif
 #include "TMCG_SecretKey.hh"
 #include "TMCG_PublicKey.hh"
 
@@ -44,7 +51,7 @@ TMCG_SecretKey::TMCG_SecretKey
 
 TMCG_SecretKey::TMCG_SecretKey
 	(const std::string& n, const std::string& e,
-	unsigned long int keysize, bool nizk_key):
+	const unsigned long int keysize, const bool nizk_key):
 		name(n), email(e)
 {
 	mpz_init(m), mpz_init(y), mpz_init(p), mpz_init(q);
@@ -69,7 +76,7 @@ TMCG_SecretKey::TMCG_SecretKey
 		name(that.name), email(that.email), type(that.type),
 		nizk(that.nizk), sig(that.sig)
 {
-	mpz_init_set(m, that.m), mpz_init_set(y, that.m),
+	mpz_init_set(m, that.m), mpz_init_set(y, that.y),
 		mpz_init_set(p, that.p), mpz_init_set(q, that.q);
 	mpz_init_set(y1, that.y1), mpz_init_set(m1pq, that.m1pq),
 		mpz_init_set(gcdext_up, that.gcdext_up),
@@ -92,7 +99,7 @@ TMCG_SecretKey& TMCG_SecretKey::operator =
 }
 
 void TMCG_SecretKey::generate
-	(unsigned long int keysize, bool nizk_key)
+	(const unsigned long int keysize, const bool nizk_key)
 {
 	mpz_t foo, bar;
 	
@@ -107,7 +114,7 @@ void TMCG_SecretKey::generate
 		t << "TMCG/RABIN_" << keysize;
 	type = t.str();
 	
-	// generate appropriate primes for RABIN encryption with SAEP
+	// generate appropriate primes for RABIN encryption with SAEP [Bo01]
 	do
 	{
 		// choose a random safe prime p, but with fixed size (n/2 + 1) bit
@@ -145,19 +152,21 @@ void TMCG_SecretKey::generate
 	while ((mpz_jacobi(y, m) != 1) || mpz_qrmn_p(y, p, q, m));
 	
 	// pre-compute non-persistent values
-	precompute();
+	bool ret = precompute();
+	assert(ret);
 
 	// Rosario Gennaro, Daniele Micciancio, Tal Rabin:
 	// 'An Efficient Non-Interactive Statistical Zero-Knowledge
 	// Proof System for Quasi-Safe Prime Products',
 	// 5th ACM Conference on Computer and Communication Security, CCS 1998
 	
-	// STAGE1/2: m = p^i * q^j, p and q prime
-	// STAGE3: y \in NQR^\circ_m
+	// STAGE1/2: m = p^i * q^j, p and q prime, i > 0, j > 0 [Sc98]; 
+	//   special m = p^1 * q^1, p and q prime and conqruent 3 mod 4 [GMR98]
+	// STAGE3: y \in NQR^\circ_m [Sc98]
 	std::ostringstream nizk2, input;
 	input << m << "^" << y, nizk2 << "nzk^";
 	size_t mnsize = mpz_sizeinbase(m, 2L) / 8;
-	char *mn = new char[mnsize];
+	unsigned char *mn = new unsigned char[mnsize];
 	
 	// STAGE1: m Square Free
 	// soundness error probability \le d^{-TMCG_KEY_NIZK_STAGE1}
@@ -167,7 +176,7 @@ void TMCG_SecretKey::generate
 		// common random number foo \in Z^*_m (build from hash function g)
 		do
 		{
-			g(mn, mnsize, (input.str()).c_str(), (input.str()).length());
+			g(mn, mnsize, (unsigned char*)(input.str()).c_str(), (input.str()).length());
 			mpz_import(foo, 1, -1, mnsize, 1, 0, mn);
 			mpz_mod(foo, foo, m);
 			mpz_gcd(bar, foo, m);
@@ -190,7 +199,7 @@ void TMCG_SecretKey::generate
 		// common random number foo \in Z^*_m (build from hash function g)
 		do
 		{
-			g(mn, mnsize, (input.str()).c_str(), (input.str()).length());
+			g(mn, mnsize, (unsigned char*)(input.str()).c_str(), (input.str()).length());
 			mpz_import(foo, 1, -1, mnsize, 1, 0, mn);
 			mpz_mod(foo, foo, m);
 			mpz_gcd(bar, foo, m);
@@ -234,7 +243,7 @@ void TMCG_SecretKey::generate
 		// common random number foo \in Z^\circ_m (build from hash function g)
 		do
 		{
-			g(mn, mnsize, (input.str()).c_str(), (input.str()).length());
+			g(mn, mnsize, (unsigned char*)(input.str()).c_str(), (input.str()).length());
 			mpz_import(foo, 1, -1, mnsize, 1, 0, mn);
 			mpz_mod(foo, foo, m);
 			input << foo;
@@ -267,31 +276,40 @@ void TMCG_SecretKey::generate
 		(repl.str()).length() + TMCG_KEYID_SIZE, keyid());
 }
 
-void TMCG_SecretKey::precompute
+bool TMCG_SecretKey::precompute
 	()
 {
 	// pre-compute non-persistent values
 	mpz_t foo;
 	mpz_init(foo);
-	
-	ret = mpz_invert(y1, y, m);
-	assert(ret);
-	mpz_sub(foo, m, p);
-	mpz_sub(foo, foo, q);
-	mpz_add_ui(foo, foo, 1L);
-	ret = mpz_invert(m1pq, m, foo);
-	assert(ret);
-	mpz_gcdext(foo, gcdext_up, gcdext_vq, p, q);
-	assert(mpz_cmp_ui(foo, 1L) == 0);
-	mpz_mul(gcdext_up, gcdext_up, p);
-	mpz_mul(gcdext_vq, gcdext_vq, q);
-	mpz_set(pa1d4, p), mpz_set(qa1d4, q);
-	mpz_add_ui(pa1d4, pa1d4, 1L);
-	mpz_add_ui(qa1d4, qa1d4, 1L);
-	mpz_fdiv_q_2exp(pa1d4, pa1d4, 2L);
-	mpz_fdiv_q_2exp(qa1d4, qa1d4, 2L);
-	
-	mpz_clear(foo);
+
+	try
+	{
+		if (!mpz_invert(y1, y, m))
+			throw false;
+		mpz_sub(foo, m, p);
+		mpz_sub(foo, foo, q);
+		mpz_add_ui(foo, foo, 1L);
+		if (!mpz_invert(m1pq, m, foo))
+			throw false;
+		mpz_gcdext(foo, gcdext_up, gcdext_vq, p, q);
+		if (mpz_cmp_ui(foo, 1L))
+			throw false;
+		mpz_mul(gcdext_up, gcdext_up, p);
+		mpz_mul(gcdext_vq, gcdext_vq, q);
+		mpz_set(pa1d4, p), mpz_set(qa1d4, q);
+		mpz_add_ui(pa1d4, pa1d4, 1L);
+		mpz_add_ui(qa1d4, qa1d4, 1L);
+		mpz_fdiv_q_2exp(pa1d4, pa1d4, 2L);
+		mpz_fdiv_q_2exp(qa1d4, qa1d4, 2L);
+
+		throw true;
+	}
+	catch (bool return_value)
+	{
+		mpz_clear(foo);
+		return return_value;
+	}	
 }
 
 bool TMCG_SecretKey::check
@@ -316,7 +334,7 @@ std::string TMCG_SecretKey::selfid
 }
 
 std::string TMCG_SecretKey::keyid
-	(size_t size) const
+	(const size_t size) const
 {
 	TMCG_PublicKey pub(*this);
 	return pub.keyid(size);
@@ -393,7 +411,8 @@ bool TMCG_SecretKey::import
 		sig = s;
 		
 		// pre-compute non-persistent values
-		precompute();
+		if (!precompute())
+			throw false;
 		
 		throw true;
 	}
@@ -404,20 +423,23 @@ bool TMCG_SecretKey::import
 }
 
 bool TMCG_SecretKey::decrypt
-	(char* value, std::string s) const
+	(unsigned char* value, std::string s) const
 {
 	mpz_t vdata, vroot[4];
 	size_t rabin_s2 = 2 * TMCG_SAEP_S0;
 	size_t rabin_s1 = (mpz_sizeinbase(m, 2L) / 8) - rabin_s2;
 	
-	assert(rabin_s2 < (mpz_sizeinbase(m, 2L) / 16));
-	assert(rabin_s2 < rabin_s1);
-	assert(TMCG_SAEP_S0 < (mpz_sizeinbase(m, 2L) / 32));
+	if (rabin_s2 >= (mpz_sizeinbase(m, 2L) / 16))
+		return false;
+	if (rabin_s2 >= rabin_s1)
+		return false;
+	if (TMCG_SAEP_S0 >= (mpz_sizeinbase(m, 2L) / 32))
+		return false;
 	
-	char *yy = new char[rabin_s2 + rabin_s1 + 1024];
-	char *r = new char[rabin_s1];
-	char *Mt = new char[rabin_s2];
-	char *g12 = new char[rabin_s2];
+	unsigned char *yy = new unsigned char[rabin_s2 + rabin_s1 + 1024];
+	unsigned char *r = new unsigned char[rabin_s1];
+	unsigned char *Mt = new unsigned char[rabin_s2];
+	unsigned char *g12 = new unsigned char[rabin_s2];
 	mpz_init(vdata), mpz_init(vroot[0]), mpz_init(vroot[1]),
 		mpz_init(vroot[2]), mpz_init(vroot[3]);
 	try
@@ -448,17 +470,17 @@ bool TMCG_SecretKey::decrypt
 			{
 				size_t cnt = 1;
 				mpz_export(yy, &cnt, -1, rabin_s2 + rabin_s1, 1, 0, vroot[k]);
-				std::memcpy(Mt, yy, rabin_s2);
-				std::memcpy(r, yy + rabin_s2, rabin_s1);
+				memcpy(Mt, yy, rabin_s2);
+				memcpy(r, yy + rabin_s2, rabin_s1);
 				g(g12, rabin_s2, r, rabin_s1);
 				
 				for (size_t i = 0; i < rabin_s2; i++)
 					Mt[i] ^= g12[i];
 				
-				std::memset(g12, 0, TMCG_SAEP_S0);
-				if (std::memcmp(Mt + TMCG_SAEP_S0, g12, TMCG_SAEP_S0) == 0)
+				memset(g12, 0, TMCG_SAEP_S0);
+				if (memcmp(Mt + TMCG_SAEP_S0, g12, TMCG_SAEP_S0) == 0)
 				{
-					std::memcpy(value, Mt, TMCG_SAEP_S0);
+					memcpy(value, Mt, TMCG_SAEP_S0);
 					throw true;
 				}
 			}
@@ -485,33 +507,33 @@ std::string TMCG_SecretKey::sign
 	
 	assert(mpz_sizeinbase(m, 2L) > (mnsize * 8));
 	assert(mnsize > (mdsize + TMCG_PRAB_K0));
+	assert((mnsize - mdsize) >= TMCG_PRAB_K0);
 	
 	// WARNING: This is only a probabilistic algorithm (Rabin's signature scheme),
 	// however, it should work with only a few iterations. Additionally the scheme
-	// PRab from [Bellare, Rogaway: The Exact Security of Digital Signatures]
-	// was implemented to increase the security.
+	// PRab [BR96] was implemented to increase the security of digital signatures.
 	do
 	{
-		char *r = new char[TMCG_PRAB_K0];
-		gcry_randomize((unsigned char*)r, TMCG_PRAB_K0, GCRY_STRONG_RANDOM);
+		unsigned char *r = new unsigned char[TMCG_PRAB_K0];
+		gcry_randomize(r, TMCG_PRAB_K0, GCRY_STRONG_RANDOM);
 		
-		char *Mr = new char[data.length() + TMCG_PRAB_K0];
-		std::memcpy(Mr, data.c_str(), data.length());
-		std::memcpy(Mr + data.length(), r, TMCG_PRAB_K0);
+		unsigned char *Mr = new unsigned char[data.length() + TMCG_PRAB_K0];
+		memcpy(Mr, data.c_str(), data.length());
+		memcpy(Mr + data.length(), r, TMCG_PRAB_K0);
 		
-		char *w = new char[mdsize];
+		unsigned char *w = new unsigned char[mdsize];
 		h(w, Mr, data.length() + TMCG_PRAB_K0);
 		
-		char *g12 = new char[mnsize];
+		unsigned char *g12 = new unsigned char[mnsize];
 		g(g12, mnsize - mdsize, w, mdsize);
 		
 		for (size_t i = 0; i < TMCG_PRAB_K0; i++)
 			r[i] ^= g12[i];
 		
-		char *yy = new char[mnsize];
-		std::memcpy(yy, w, mdsize);
-		std::memcpy(yy + mdsize, r, TMCG_PRAB_K0);
-		std::memcpy(yy + mdsize + TMCG_PRAB_K0, g12 + TMCG_PRAB_K0,
+		unsigned char *yy = new unsigned char[mnsize];
+		memcpy(yy, w, mdsize);
+		memcpy(yy + mdsize, r, TMCG_PRAB_K0);
+		memcpy(yy + mdsize + TMCG_PRAB_K0, g12 + TMCG_PRAB_K0,
 			mnsize - mdsize - TMCG_PRAB_K0);
 		mpz_import(foo, 1, -1, mnsize, 1, 0, yy);
 		
@@ -531,7 +553,7 @@ std::string TMCG_SecretKey::sign
 }
 
 std::string TMCG_SecretKey::encrypt
-	(const char* value) const
+	(const unsigned char* value) const
 {
 	TMCG_PublicKey pub(*this);
 	return pub.encrypt(value);
