@@ -37,13 +37,13 @@ void h
 	gcry_md_hash_buffer(TMCG_GCRY_MD_ALGO, output, input, size);
 }
 
-/* hash function g() (The design is based on the ideas of [BR95].) */
+/* hash function g() (The design is based on ideas from [BR95].) */
 void g
 	(unsigned char *output, const size_t osize,
 	const unsigned char *input, const size_t isize)
 {
 	size_t mdsize = gcry_md_get_algo_dlen(TMCG_GCRY_MD_ALGO);
-	// hopefully this truncation size does not match the border
+	// hopefully, size of truncation does not match the border
 	// of chaining variables in the compression function of h
 	size_t usesize = (mdsize / 4) + 1;
 	size_t times = (osize / usesize) + 1;
@@ -71,13 +71,14 @@ void g
 size_t mpz_shash_len
 	()
 {
+// FIXME: check wheter this length in bit matches the used subgroup size
 	return gcry_md_get_algo_dlen(TMCG_GCRY_MD_ALGO);
 }
 
 void mpz_shash
 	(mpz_ptr r, std::string input)
 {
-	size_t hash_size = gcry_md_get_algo_dlen(TMCG_GCRY_MD_ALGO);
+	size_t hash_size = mpz_shash_len();
 	unsigned char *digest = new unsigned char[hash_size];
 	char *hex_digest = new char[(2 * hash_size) + 1];
 	
