@@ -112,11 +112,11 @@ void start_instance
 			stop_clock();
 			std::cout << "P_" << whoami << ": " << elapsed_time() << std::endl;
 
-			// create an instance of DKG
+			// create an instance of DKG (without using very strong randomness)
 			GennaroJareckiKrawczykRabinDKG *dkg;
 			std::cout << "P_" << whoami << ": GennaroJareckiKrawczykRabinDKG(" << N << ", " << T << ", " << whoami << ", ...)" << std::endl;
 			dkg = new GennaroJareckiKrawczykRabinDKG(N, T, whoami,
-				vtmf->p, vtmf->q, vtmf->g, vtmf->h);
+				vtmf->p, vtmf->q, vtmf->g, vtmf->h, TMCG_DDH_SIZE, TMCG_DLSE_SIZE, false);
 			assert(dkg->CheckGroup());
 
 			// create asynchronous authenticated unicast channels
